package api_server

import (
	"net/url"
	"strconv"

	"github.com/kumahq/kuma/pkg/api-server/types"

	"github.com/emicklei/go-restful"

	"github.com/kumahq/kuma/pkg/core/resources/model"
)

const maxPageSize = 1000
const defaultPageSize = 100

type page struct {
	size   int
	offset string
}

func pagination(request *restful.Request) (page, error) {
	pageSize := defaultPageSize
	if request.QueryParameter("size") != "" {
		p, err := strconv.Atoi(request.QueryParameter("size"))
		if err != nil {
			return page{}, types.InvalidPageSize
		}
		pageSize = p
		if pageSize > maxPageSize {
			return page{}, types.NewMaxPageSizeExceeded(pageSize, maxPageSize)
		}
	}
	offset := request.QueryParameter("offset")
	return page{
		size:   pageSize,
		offset: offset,
	}, nil
}

func nextLink(request *restful.Request, list model.ResourceList) *string {
	if list.GetPagination().NextOffset == "" {
		return nil
	}

	query := request.Request.URL.Query()
	query.Set("offset", list.GetPagination().NextOffset)

	nextURL := &url.URL{}
	if request.Request.TLS == nil {
		nextURL.Scheme = "http"
	} else {
		nextURL.Scheme = "https"
	}
	nextURL.Host = request.Request.Host
	nextURL.Path = request.Request.URL.Path
	nextURL.RawQuery = query.Encode()
	urlString := nextURL.String()
	return &urlString
}
